@@ -26,6 +26,8 @@
 #include "forest/ForestTrainers.h"
 #include "RcppUtilities.h"
 
+using namespace grf;
+
 // [[Rcpp::export]]
 Rcpp::List causal_train(Rcpp::NumericMatrix train_matrix,
                         Eigen::SparseMatrix<double> sparse_train_matrix,
@@ -39,6 +41,7 @@ Rcpp::List causal_train(Rcpp::NumericMatrix train_matrix,
                         double sample_fraction,
                         bool honesty,
                         double honesty_fraction,
+                        bool prune_empty_leaves,
                         size_t ci_group_size,
                         double reduced_form_weight,
                         double alpha,
@@ -49,9 +52,9 @@ Rcpp::List causal_train(Rcpp::NumericMatrix train_matrix,
                         bool compute_oob_predictions,
                         unsigned int num_threads,
                         unsigned int seed) {
-  ForestTrainer trainer = ForestTrainers::instrumental_trainer(reduced_form_weight, stabilize_splits);
+  ForestTrainer trainer = instrumental_trainer(reduced_form_weight, stabilize_splits);
 
-  Data* data = RcppUtilities::convert_data(train_matrix, sparse_train_matrix);
+  std::unique_ptr<Data> data = RcppUtilities::convert_data(train_matrix, sparse_train_matrix);
   data->set_outcome_index(outcome_index - 1);
   data->set_treatment_index(treatment_index - 1);
   data->set_instrument_index(treatment_index - 1);
@@ -61,16 +64,15 @@ Rcpp::List causal_train(Rcpp::NumericMatrix train_matrix,
   data->sort();
 
   ForestOptions options(num_trees, ci_group_size, sample_fraction, mtry, min_node_size, honesty,
-                        honesty_fraction, alpha, imbalance_penalty, num_threads, seed, clusters, samples_per_cluster);
-  Forest forest = trainer.train(data, options);
+                        honesty_fraction, prune_empty_leaves, alpha, imbalance_penalty, num_threads, seed, clusters, samples_per_cluster);
+  Forest forest = trainer.train(*data, options);
 
   std::vector<Prediction> predictions;
   if (compute_oob_predictions) {
-    ForestPredictor predictor = ForestPredictors::instrumental_predictor(num_threads);
-    predictions = predictor.predict_oob(forest, data, false);
+    ForestPredictor predictor = instrumental_predictor(num_threads);
+    predictions = predictor.predict_oob(forest, *data, false);
   }
 
-  delete data;
   return RcppUtilities::create_forest_object(forest, predictions);
 }
 
@@ -85,20 +87,18 @@ Rcpp::List causal_predict(Rcpp::List forest_object,
                           Eigen::SparseMatrix<double> sparse_test_matrix,
                           unsigned int num_threads,
                           bool estimate_variance) {
-  Data* train_data = RcppUtilities::convert_data(train_matrix, sparse_train_matrix);
+  std::unique_ptr<Data> train_data = RcppUtilities::convert_data(train_matrix, sparse_train_matrix);
   train_data->set_outcome_index(outcome_index - 1);
   train_data->set_treatment_index(treatment_index - 1);
   train_data->set_instrument_index(treatment_index - 1);
-  Data* data = RcppUtilities::convert_data(test_matrix, sparse_test_matrix);
+  std::unique_ptr<Data> data = RcppUtilities::convert_data(test_matrix, sparse_test_matrix);
 
   Forest forest = RcppUtilities::deserialize_forest(forest_object);
 
-  ForestPredictor predictor = ForestPredictors::instrumental_predictor(num_threads);
-  std::vector<Prediction> predictions = predictor.predict(forest, train_data, data, estimate_variance);
+  ForestPredictor predictor = instrumental_predictor(num_threads);
+  std::vector<Prediction> predictions = predictor.predict(forest, *train_data, *data, estimate_variance);
   Rcpp::List result = RcppUtilities::create_prediction_object(predictions);
 
-  delete train_data;
-  delete data;
   return result;
 }
 
@@ -110,35 +110,35 @@ Rcpp::List causal_predict_oob(Rcpp::List forest_object,
                               size_t treatment_index,
                               unsigned int num_threads,
                               bool estimate_variance) {
-  Data* data = RcppUtilities::convert_data(train_matrix, sparse_train_matrix);
+  std::unique_ptr<Data> data = RcppUtilities::convert_data(train_matrix, sparse_train_matrix);
   data->set_outcome_index(outcome_index - 1);
   data->set_treatment_index(treatment_index - 1);
   data->set_instrument_index(treatment_index - 1);
 
   Forest forest = RcppUtilities::deserialize_forest(forest_object);
 
-  ForestPredictor predictor = ForestPredictors::instrumental_predictor(num_threads);
-  std::vector<Prediction> predictions = predictor.predict_oob(forest, data, estimate_variance);
+  ForestPredictor predictor = instrumental_predictor(num_threads);
+  std::vector<Prediction> predictions = predictor.predict_oob(forest, *data, estimate_variance);
   Rcpp::List result = RcppUtilities::create_prediction_object(predictions);
 
-  delete data;
   return result;
 }
 
 // [[Rcpp::export]]
 Rcpp::List ll_causal_predict(Rcpp::List forest,
-                                 Rcpp::NumericMatrix input_data,
-                                 Rcpp::NumericMatrix training_data,
-                                 Eigen::SparseMatrix<double> sparse_input_data,
-                                 Eigen::SparseMatrix<double> sparse_training_data,
-                                 size_t outcome_index,
-                                 size_t treatment_index,
-                                 std::vector<double> lambdas,
-                                 bool use_unweighted_penalty,
-                                 std::vector<size_t> linear_correction_variables,
-                                 unsigned int num_threads) {
-  Data* test_data = RcppUtilities::convert_data(input_data, sparse_input_data);
-  Data* train_data = RcppUtilities::convert_data(training_data, sparse_training_data);
+                             Rcpp::NumericMatrix input_data,
+                             Rcpp::NumericMatrix training_data,
+                             Eigen::SparseMatrix<double> sparse_input_data,
+                             Eigen::SparseMatrix<double> sparse_training_data,
+                             size_t outcome_index,
+                             size_t treatment_index,
+                             std::vector<double> lambdas,
+                             bool use_weighted_penalty,
+                             std::vector<size_t> linear_correction_variables,
+                             unsigned int num_threads,
+                             bool estimate_variance) {
+  std::unique_ptr<Data> data = RcppUtilities::convert_data(input_data, sparse_input_data);
+  std::unique_ptr<Data> train_data = RcppUtilities::convert_data(training_data, sparse_training_data);
 
   train_data->set_outcome_index(outcome_index - 1);
   train_data->set_treatment_index(treatment_index - 1);
@@ -146,13 +146,11 @@ Rcpp::List ll_causal_predict(Rcpp::List forest,
 
   Forest deserialized_forest = RcppUtilities::deserialize_forest(forest);
 
-  ForestPredictor predictor = ForestPredictors::ll_causal_predictor(num_threads, lambdas, use_unweighted_penalty,
+  ForestPredictor predictor = ll_causal_predictor(num_threads, lambdas, use_weighted_penalty,
                                                                  linear_correction_variables);
-  std::vector<Prediction> predictions = predictor.predict(deserialized_forest, train_data, test_data, false);
+  std::vector<Prediction> predictions = predictor.predict(deserialized_forest, *train_data, *data, estimate_variance);
   Rcpp::List result = RcppUtilities::create_prediction_object(predictions);
 
-  delete train_data;
-  delete test_data;
   return result;
 }
 
@@ -163,10 +161,11 @@ Rcpp::List ll_causal_predict_oob(Rcpp::List forest,
                                      size_t outcome_index,
                                      size_t treatment_index,
                                      std::vector<double> lambdas,
-                                     bool use_unweighted_penalty,
+                                     bool use_weighted_penalty,
                                      std::vector<size_t> linear_correction_variables,
-                                     unsigned int num_threads) {
-  Data* data = RcppUtilities::convert_data(input_data, sparse_input_data);
+                                     unsigned int num_threads,
+                                     bool estimate_variance) {
+  std::unique_ptr<Data> data = RcppUtilities::convert_data(input_data, sparse_input_data);
 
   data->set_outcome_index(outcome_index - 1);
   data->set_treatment_index(treatment_index - 1);
@@ -174,11 +173,10 @@ Rcpp::List ll_causal_predict_oob(Rcpp::List forest,
 
   Forest deserialized_forest = RcppUtilities::deserialize_forest(forest);
 
-  ForestPredictor predictor = ForestPredictors::ll_causal_predictor(num_threads, lambdas, use_unweighted_penalty,
+  ForestPredictor predictor = ll_causal_predictor(num_threads, lambdas, use_weighted_penalty,
                                                                  linear_correction_variables);
-  std::vector<Prediction> predictions = predictor.predict_oob(deserialized_forest, data, false);
+  std::vector<Prediction> predictions = predictor.predict_oob(deserialized_forest, *data, estimate_variance);
   Rcpp::List result = RcppUtilities::create_prediction_object(predictions);
 
-  delete data;
   return result;
 }
