@@ -9,6 +9,8 @@
 #include "forest/ForestTrainers.h"
 #include "RcppUtilities.h"
 
+using namespace grf;
+
 // [[Rcpp::export]]
 Rcpp::List quantile_train(std::vector<double> quantiles,
                           bool regression_splits,
@@ -21,6 +23,7 @@ Rcpp::List quantile_train(std::vector<double> quantiles,
                           double sample_fraction,
                           bool honesty,
                           double honesty_fraction,
+                          bool prune_empty_leaves,
                           size_t ci_group_size,
                           double alpha,
                           double imbalance_penalty,
@@ -29,18 +32,17 @@ Rcpp::List quantile_train(std::vector<double> quantiles,
                           int num_threads,
                           unsigned int seed) {
   ForestTrainer trainer = regression_splits
-      ? ForestTrainers::regression_trainer()
-      : ForestTrainers::quantile_trainer(quantiles);
+      ? regression_trainer()
+      : quantile_trainer(quantiles);
 
-  Data* data = RcppUtilities::convert_data(train_matrix, sparse_train_matrix);
+  std::unique_ptr<Data> data = RcppUtilities::convert_data(train_matrix, sparse_train_matrix);
   data->set_outcome_index(outcome_index - 1);
   data->sort();
 
   ForestOptions options(num_trees, ci_group_size, sample_fraction, mtry, min_node_size, honesty,
-      honesty_fraction, alpha, imbalance_penalty, num_threads, seed, clusters, samples_per_cluster);
-  Forest forest = trainer.train(data, options);
+      honesty_fraction, prune_empty_leaves, alpha, imbalance_penalty, num_threads, seed, clusters, samples_per_cluster);
+  Forest forest = trainer.train(*data, options);
 
-  delete data;
   return RcppUtilities::create_forest_object(forest, std::vector<Prediction>());
 }
 
@@ -53,18 +55,16 @@ Rcpp::NumericMatrix quantile_predict(Rcpp::List forest_object,
                                      Rcpp::NumericMatrix test_matrix,
                                      Eigen::SparseMatrix<double> sparse_test_matrix,
                                      unsigned int num_threads) {
-  Data* train_data = RcppUtilities::convert_data(train_matrix, sparse_train_matrix);
-  Data* data = RcppUtilities::convert_data(test_matrix, sparse_test_matrix);
+  std::unique_ptr<Data> train_data = RcppUtilities::convert_data(train_matrix, sparse_train_matrix);
+  std::unique_ptr<Data> data = RcppUtilities::convert_data(test_matrix, sparse_test_matrix);
   train_data->set_outcome_index(outcome_index - 1);
 
   Forest forest = RcppUtilities::deserialize_forest(forest_object);
 
-  ForestPredictor predictor = ForestPredictors::quantile_predictor(num_threads, quantiles);
-  std::vector<Prediction> predictions = predictor.predict(forest, train_data, data, false);
+  ForestPredictor predictor = quantile_predictor(num_threads, quantiles);
+  std::vector<Prediction> predictions = predictor.predict(forest, *train_data, *data, false);
   Rcpp::NumericMatrix result = RcppUtilities::create_prediction_matrix(predictions);
 
-  delete train_data;
-  delete data;
   return result;
 }
 
@@ -75,15 +75,14 @@ Rcpp::NumericMatrix quantile_predict_oob(Rcpp::List forest_object,
                                          Eigen::SparseMatrix<double> sparse_train_matrix,
                                          size_t outcome_index,
                                          unsigned int num_threads) {
-  Data* data = RcppUtilities::convert_data(train_matrix, sparse_train_matrix);
+  std::unique_ptr<Data> data = RcppUtilities::convert_data(train_matrix, sparse_train_matrix);
   data->set_outcome_index(outcome_index - 1);
 
   Forest forest = RcppUtilities::deserialize_forest(forest_object);
 
-  ForestPredictor predictor = ForestPredictors::quantile_predictor(num_threads, quantiles);
-  std::vector<Prediction> predictions = predictor.predict_oob(forest, data, false);
+  ForestPredictor predictor = quantile_predictor(num_threads, quantiles);
+  std::vector<Prediction> predictions = predictor.predict_oob(forest, *data, false);
   Rcpp::NumericMatrix result = RcppUtilities::create_prediction_matrix(predictions);
 
-  delete data;
   return result;
 }
