@@ -26,6 +26,8 @@
 #include "forest/ForestTrainers.h"
 #include "RcppUtilities.h"
 
+using namespace grf;
+
 // [[Rcpp::export]]
 Rcpp::List regression_train(Rcpp::NumericMatrix train_matrix,
                             Eigen::SparseMatrix<double> sparse_train_matrix,
@@ -38,6 +40,7 @@ Rcpp::List regression_train(Rcpp::NumericMatrix train_matrix,
                             double sample_fraction,
                             bool honesty,
                             double honesty_fraction,
+                            bool prune_empty_leaves,
                             size_t ci_group_size,
                             double alpha,
                             double imbalance_penalty,
@@ -46,9 +49,9 @@ Rcpp::List regression_train(Rcpp::NumericMatrix train_matrix,
                             bool compute_oob_predictions,
                             unsigned int num_threads,
                             unsigned int seed) {
-  ForestTrainer trainer = ForestTrainers::regression_trainer();
+  ForestTrainer trainer = regression_trainer();
 
-  Data* data = RcppUtilities::convert_data(train_matrix, sparse_train_matrix);
+  std::unique_ptr<Data> data = RcppUtilities::convert_data(train_matrix, sparse_train_matrix);
   data->set_outcome_index(outcome_index - 1);
   if(use_sample_weights) {
       data->set_weight_index(sample_weight_index - 1);
@@ -56,16 +59,15 @@ Rcpp::List regression_train(Rcpp::NumericMatrix train_matrix,
   data->sort();
 
   ForestOptions options(num_trees, ci_group_size, sample_fraction, mtry, min_node_size, honesty,
-      honesty_fraction, alpha, imbalance_penalty, num_threads, seed, clusters, samples_per_cluster);
-  Forest forest = trainer.train(data, options);
+      honesty_fraction, prune_empty_leaves, alpha, imbalance_penalty, num_threads, seed, clusters, samples_per_cluster);
+  Forest forest = trainer.train(*data, options);
 
   std::vector<Prediction> predictions;
   if (compute_oob_predictions) {
-    ForestPredictor predictor = ForestPredictors::regression_predictor(num_threads);
-    predictions = predictor.predict_oob(forest, data, false);
+    ForestPredictor predictor = regression_predictor(num_threads);
+    predictions = predictor.predict_oob(forest, *data, false);
   }
 
-  delete data;
   return RcppUtilities::create_forest_object(forest, predictions);
 }
 
@@ -78,17 +80,15 @@ Rcpp::List regression_predict(Rcpp::List forest_object,
                               Eigen::SparseMatrix<double> sparse_test_matrix,
                               unsigned int num_threads,
                               unsigned int estimate_variance) {
-  Data* train_data = RcppUtilities::convert_data(train_matrix, sparse_train_matrix);
+  std::unique_ptr<Data> train_data = RcppUtilities::convert_data(train_matrix, sparse_train_matrix);
   train_data->set_outcome_index(outcome_index - 1);
 
-  Data* data = RcppUtilities::convert_data(test_matrix, sparse_test_matrix);
+  std::unique_ptr<Data> data = RcppUtilities::convert_data(test_matrix, sparse_test_matrix);
   Forest forest = RcppUtilities::deserialize_forest(forest_object);
 
-  ForestPredictor predictor = ForestPredictors::regression_predictor(num_threads);
-  std::vector<Prediction> predictions = predictor.predict(forest, train_data, data, estimate_variance);
+  ForestPredictor predictor = regression_predictor(num_threads);
+  std::vector<Prediction> predictions = predictor.predict(forest, *train_data, *data, estimate_variance);
 
-  delete train_data;
-  delete data;
   return RcppUtilities::create_prediction_object(predictions);
 }
 
@@ -99,16 +99,15 @@ Rcpp::List regression_predict_oob(Rcpp::List forest_object,
                                   size_t outcome_index,
                                   unsigned int num_threads,
                                   bool estimate_variance) {
-  Data* data = RcppUtilities::convert_data(train_matrix, sparse_train_matrix);
+  std::unique_ptr<Data> data = RcppUtilities::convert_data(train_matrix, sparse_train_matrix);
   data->set_outcome_index(outcome_index - 1);
 
   Forest forest = RcppUtilities::deserialize_forest(forest_object);
 
-  ForestPredictor predictor = ForestPredictors::regression_predictor(num_threads);
-  std::vector<Prediction> predictions = predictor.predict_oob(forest, data, estimate_variance);
+  ForestPredictor predictor = regression_predictor(num_threads);
+  std::vector<Prediction> predictions = predictor.predict_oob(forest, *data, estimate_variance);
 
   Rcpp::List result = RcppUtilities::create_prediction_object(predictions);
-  delete data;
   return result;
 }
 
@@ -124,19 +123,17 @@ Rcpp::List ll_regression_predict(Rcpp::List forest,
                                 std::vector<size_t> linear_correction_variables,
                                 unsigned int num_threads,
                                 bool estimate_variance) {
-  Data* train_data = RcppUtilities::convert_data(train_matrix, sparse_train_matrix);
+  std::unique_ptr<Data> train_data = RcppUtilities::convert_data(train_matrix, sparse_train_matrix);
   train_data->set_outcome_index(outcome_index - 1);
-  Data* data = RcppUtilities::convert_data(test_matrix, sparse_test_matrix);
+  std::unique_ptr<Data> data = RcppUtilities::convert_data(test_matrix, sparse_test_matrix);
 
   Forest deserialized_forest = RcppUtilities::deserialize_forest(forest);
 
-  ForestPredictor predictor = ForestPredictors::ll_regression_predictor(num_threads,
+  ForestPredictor predictor = ll_regression_predictor(num_threads,
       lambdas, weight_penalty, linear_correction_variables);
-  std::vector<Prediction> predictions = predictor.predict(deserialized_forest, train_data, data, estimate_variance);
+  std::vector<Prediction> predictions = predictor.predict(deserialized_forest, *train_data, *data, estimate_variance);
   Rcpp::List result = RcppUtilities::create_prediction_object(predictions);
 
-  delete train_data;
-  delete data;
   return result;
 }
 
@@ -150,17 +147,15 @@ Rcpp::List ll_regression_predict_oob(Rcpp::List forest,
                                     std::vector<size_t> linear_correction_variables,
                                     unsigned int num_threads,
                                     bool estimate_variance) {
-  Data* data = RcppUtilities::convert_data(train_matrix, sparse_train_matrix);
+  std::unique_ptr<Data> data = RcppUtilities::convert_data(train_matrix, sparse_train_matrix);
   data->set_outcome_index(outcome_index - 1);
 
   Forest deserialized_forest = RcppUtilities::deserialize_forest(forest);
 
-  ForestPredictor predictor = ForestPredictors::ll_regression_predictor(num_threads,
+  ForestPredictor predictor = ll_regression_predictor(num_threads,
       lambdas, weight_penalty, linear_correction_variables);
-  std::vector<Prediction> predictions = predictor.predict_oob(deserialized_forest, data, estimate_variance);
+  std::vector<Prediction> predictions = predictor.predict_oob(deserialized_forest, *data, estimate_variance);
   Rcpp::List result = RcppUtilities::create_prediction_object(predictions);
 
-  delete data;
   return result;
 }
-
