@@ -22,24 +22,19 @@
 
 #include "Data.h"
 
+namespace grf {
+
 Data::Data() :
     num_rows(0),
     num_cols(0),
-    external_data(true),
-    index_data(0),
+    index_data(),
     max_num_unique_values(0),
     outcome_index(),
     treatment_index(),
     instrument_index(),
     weight_index() {}
 
-Data::~Data() {
-  if (index_data != 0) {
-    delete[] index_data;
-  }
-}
-
-bool Data::load_from_file(std::string filename) {
+bool Data::load_from_file(const std::string& filename) {
   bool result;
 
   // Open input file
@@ -65,24 +60,24 @@ bool Data::load_from_file(std::string filename) {
   input_file.open(filename);
 
   // Find out if comma, semicolon or whitespace seperated and call appropriate method
-  if (first_line.find(",") != std::string::npos) {
+  if (first_line.find(',') != std::string::npos) {
     result = load_from_other_file(input_file, first_line, ',');
-  } else if (first_line.find(";") != std::string::npos) {
+  } else if (first_line.find(';') != std::string::npos) {
     result = load_from_other_file(input_file, first_line, ';');
   } else {
     result = load_from_whitespace_file(input_file, first_line);
   }
 
-  external_data = false;
   input_file.close();
   return result;
 }
 
-bool Data::load_from_whitespace_file(std::ifstream& input_file, std::string first_line) {
+bool Data::load_from_whitespace_file(std::ifstream& input_file,
+                                     const std::string& first_line) {
   // Read the first line to determine the number of columns.
   std::string dummy_token;
   std::stringstream first_line_stream(first_line);
-  while (first_line_stream >> dummy_token)  {
+  while (first_line_stream >> dummy_token) {
     num_cols++;
   }
 
@@ -110,7 +105,9 @@ bool Data::load_from_whitespace_file(std::ifstream& input_file, std::string firs
   return error;
 }
 
-bool Data::load_from_other_file(std::ifstream& input_file, std::string first_line, char seperator) {
+bool Data::load_from_other_file(std::ifstream& input_file,
+                                const std::string& first_line,
+                                char seperator) {
   // Read the first line to determine the number of columns.
   std::string dummy_token;
   std::stringstream first_line_stream(first_line);
@@ -156,14 +153,15 @@ void Data::set_instrument_index(size_t index) {
 }
 
 void Data::set_weight_index(size_t index) {
-    this->weight_index = index;
-    disallowed_split_variables.insert(index);
+  this->weight_index = index;
+  disallowed_split_variables.insert(index);
 }
 
 void Data::get_all_values(std::vector<double>& all_values, const std::vector<size_t>& samples, size_t var) const {
-  all_values.reserve(samples.size());
-  for (size_t i = 0; i < samples.size(); ++i) {
-    all_values.push_back(get(samples[i], var));
+  all_values.resize(samples.size());
+  for (size_t i = 0; i < samples.size(); i++) {
+    size_t sample = samples[i];
+    all_values[i] = get(sample, var);
   }
   std::sort(all_values.begin(), all_values.end());
   all_values.erase(unique(all_values.begin(), all_values.end()), all_values.end());
@@ -175,7 +173,7 @@ size_t Data::get_index(size_t row, size_t col) const {
 
 void Data::sort() {
   // Reserve memory
-  index_data = new size_t[num_cols * num_rows];
+  index_data.resize(num_cols * num_rows);
 
   // For all columns, get unique values and save index for each observation
   for (size_t col = 0; col < num_cols; ++col) {
@@ -190,7 +188,8 @@ void Data::sort() {
 
     // Get index of unique value
     for (size_t row = 0; row < num_rows; ++row) {
-      size_t idx = std::lower_bound(unique_values.begin(), unique_values.end(), get(row, col)) - unique_values.begin();
+      size_t idx =
+          std::lower_bound(unique_values.begin(), unique_values.end(), get(row, col)) - unique_values.begin();
       index_data[col * num_rows + row] = idx;
     }
 
@@ -235,7 +234,7 @@ double Data::get_instrument(size_t row) const {
 }
 
 double Data::get_weight(size_t row) const {
-  if(weight_index.has_value()) {
+  if (weight_index.has_value()) {
     return get(row, weight_index.value());
   } else {
     return 1.0;
@@ -245,3 +244,5 @@ double Data::get_weight(size_t row) const {
 const std::set<size_t>& Data::get_disallowed_split_variables() const {
   return disallowed_split_variables;
 }
+
+} // namespace grf
