@@ -35,33 +35,36 @@
 #include <thread>
 #include <future>
 
+namespace grf {
+
 class ForestPredictor {
 public:
   ForestPredictor(uint num_threads,
-                  std::shared_ptr<DefaultPredictionStrategy> strategy);
+                  std::unique_ptr<DefaultPredictionStrategy> strategy);
 
   ForestPredictor(uint num_threads,
-                  std::shared_ptr<OptimizedPredictionStrategy> strategy);
+                  std::unique_ptr<OptimizedPredictionStrategy> strategy);
 
   std::vector<Prediction> predict(const Forest& forest,
-                                  Data* train_data,
-                                  Data* data,
+                                  const Data& train_data,
+                                  const Data& data,
                                   bool estimate_variance) const;
 
   std::vector<Prediction> predict_oob(const Forest& forest,
-                                      Data* data,
+                                      const Data& data,
                                       bool estimate_variance) const;
 private:
   std::vector<Prediction> predict(const Forest& forest,
-                                  Data* train_data,
-                                  Data* data,
+                                  const Data& train_data,
+                                  const Data& data,
                                   bool estimate_variance,
                                   bool oob_prediction) const;
 
 private:
   TreeTraverser tree_traverser;
-  std::shared_ptr<PredictionCollector> prediction_collector;
+  std::unique_ptr<PredictionCollector> prediction_collector;
 };
 
+} // namespace grf
 
 #endif //GRF_FORESTPREDICTOR_H
