@@ -23,17 +23,29 @@
 
 #include "commons/utility.h"
 #include "ForestTrainer.h"
+#include "random/random.hpp"
 
-ForestTrainer::ForestTrainer(std::shared_ptr<RelabelingStrategy> relabeling_strategy,
-                             std::shared_ptr<SplittingRuleFactory> splitting_rule_factory,
-                             std::shared_ptr<OptimizedPredictionStrategy> prediction_strategy) :
-    tree_trainer(relabeling_strategy,
-                 splitting_rule_factory,
-                 prediction_strategy) {}
 
-const Forest ForestTrainer::train(const Data* data,
-                                  const ForestOptions& options) const {
-  size_t num_samples = data->get_num_rows();
+namespace grf {
+
+ForestTrainer::ForestTrainer(std::unique_ptr<RelabelingStrategy> relabeling_strategy,
+                             std::unique_ptr<SplittingRuleFactory> splitting_rule_factory,
+                             std::unique_ptr<OptimizedPredictionStrategy> prediction_strategy) :
+    tree_trainer(std::move(relabeling_strategy),
+                 std::move(splitting_rule_factory),
+                 std::move(prediction_strategy)) {}
+
+Forest ForestTrainer::train(const Data& data, const ForestOptions& options) const {
+  std::vector<std::unique_ptr<Tree>> trees = train_trees(data, options);
+
+  size_t num_variables = data.get_num_cols() - data.get_disallowed_split_variables().size();
+  size_t ci_group_size = options.get_ci_group_size();
+  return Forest(trees, num_variables, ci_group_size);
+}
+
+std::vector<std::unique_ptr<Tree>> ForestTrainer::train_trees(const Data& data,
+                                                              const ForestOptions& options) const {
+  size_t num_samples = data.get_num_rows();
   uint num_trees = options.get_num_trees();
 
   // Ensure that the sample fraction is not too small and honesty fraction is not too extreme.
@@ -52,10 +64,10 @@ const Forest ForestTrainer::train(const Data* data,
   std::vector<uint> thread_ranges;
   split_sequence(thread_ranges, 0, num_groups - 1, options.get_num_threads());
 
-  std::vector<std::future<std::vector<std::shared_ptr<Tree>>>> futures;
+  std::vector<std::future<std::vector<std::unique_ptr<Tree>>>> futures;
   futures.reserve(thread_ranges.size());
 
-  std::vector<std::shared_ptr<Tree>> trees;
+  std::vector<std::unique_ptr<Tree>> trees;
   trees.reserve(num_trees);
 
   for (uint i = 0; i < thread_ranges.size() - 1; ++i) {
@@ -67,28 +79,30 @@ const Forest ForestTrainer::train(const Data* data,
                                  this,
                                  start_index,
                                  num_trees_batch,
-                                 data,
+                                 std::ref(data),
                                  options));
   }
 
   for (auto& future : futures) {
-    std::vector<std::shared_ptr<Tree>> thread_trees = future.get();
-    trees.insert(trees.end(), thread_trees.begin(), thread_trees.end());
+    std::vector<std::unique_ptr<Tree>> thread_trees = future.get();
+    trees.insert(trees.end(),
+                 std::make_move_iterator(thread_trees.begin()),
+                 std::make_move_iterator(thread_trees.end()));
   }
 
-  return Forest::create(trees, options, data);
+  return trees;
 }
 
-std::vector<std::shared_ptr<Tree>> ForestTrainer::train_batch(
+std::vector<std::unique_ptr<Tree>> ForestTrainer::train_batch(
     size_t start,
     size_t num_trees,
-    const Data* data,
+    const Data& data,
     const ForestOptions& options) const {
   size_t ci_group_size = options.get_ci_group_size();
 
   std::mt19937_64 random_number_generator(options.get_random_seed() + start);
-  std::uniform_int_distribution<uint> udist;
-  std::vector<std::shared_ptr<Tree>> trees;
+  nonstd::uniform_int_distribution<uint> udist;
+  std::vector<std::unique_ptr<Tree>> trees;
 
   if (ci_group_size == 1) {
     trees.reserve(num_trees);
@@ -101,40 +115,42 @@ std::vector<std::shared_ptr<Tree>> ForestTrainer::train_batch(
     RandomSampler sampler(tree_seed, options.get_sampling_options());
 
     if (ci_group_size == 1) {
-      std::shared_ptr<Tree> tree = train_tree(data, sampler, options);
-      trees.push_back(tree);
+      std::unique_ptr<Tree> tree = train_tree(data, sampler, options);
+      trees.push_back(std::move(tree));
     } else {
-      std::vector<std::shared_ptr<Tree>> group = train_ci_group(data, sampler, options);
-      trees.insert(trees.end(), group.begin(), group.end());
+      std::vector<std::unique_ptr<Tree>> group = train_ci_group(data, sampler, options);
+      trees.insert(trees.end(),
+          std::make_move_iterator(group.begin()),
+          std::make_move_iterator(group.end()));
     }
   }
   return trees;
 }
-
-std::shared_ptr<Tree> ForestTrainer::train_tree(const Data* data,
+std::unique_ptr<Tree> ForestTrainer::train_tree(const Data& data,
                                                 RandomSampler& sampler,
                                                 const ForestOptions& options) const {
   std::vector<size_t> clusters;
-  sampler.sample_clusters(data->get_num_rows(), options.get_sample_fraction(), clusters);
+  sampler.sample_clusters(data.get_num_rows(), options.get_sample_fraction(), clusters);
   return tree_trainer.train(data, sampler, clusters, options.get_tree_options());
 }
 
-std::vector<std::shared_ptr<Tree>> ForestTrainer::train_ci_group(const Data* data,
+std::vector<std::unique_ptr<Tree>> ForestTrainer::train_ci_group(const Data& data,
                                                                  RandomSampler& sampler,
                                                                  const ForestOptions& options) const {
-  std::vector<std::shared_ptr<Tree>> trees;
+  std::vector<std::unique_ptr<Tree>> trees;
 
   std::vector<size_t> clusters;
-  sampler.sample_clusters(data->get_num_rows(), 0.5, clusters);
+  sampler.sample_clusters(data.get_num_rows(), 0.5, clusters);
 
   double sample_fraction = options.get_sample_fraction();
   for (size_t i = 0; i < options.get_ci_group_size(); ++i) {
     std::vector<size_t> cluster_subsample;
     sampler.subsample(clusters, sample_fraction * 2, cluster_subsample);
 
-    std::shared_ptr<Tree> tree = tree_trainer.train(data,
-        sampler, cluster_subsample, options.get_tree_options());
-    trees.push_back(tree);
+    std::unique_ptr<Tree> tree = tree_trainer.train(data, sampler, cluster_subsample, options.get_tree_options());
+    trees.push_back(std::move(tree));
   }
   return trees;
 }
+
+} // namespace grf
