@@ -15,8 +15,8 @@
   along with grf. If not, see <http://www.gnu.org/licenses/>.
  #-------------------------------------------------------------------------------*/
 
-#ifndef GRF_FULLPREDICTOR_H
-#define GRF_FULLPREDICTOR_H
+#ifndef GRF_DEFAULTPREDICTIONCOLLECTOR_H
+#define GRF_DEFAULTPREDICTIONCOLLECTOR_H
 
 
 #include "forest/Forest.h"
@@ -24,24 +24,27 @@
 #include "prediction/collector/SampleWeightComputer.h"
 #include "prediction/DefaultPredictionStrategy.h"
 
-class DefaultPredictionCollector: public PredictionCollector {
+namespace grf {
+
+class DefaultPredictionCollector final: public PredictionCollector {
 public:
-  DefaultPredictionCollector(std::shared_ptr<DefaultPredictionStrategy> strategy);
+  DefaultPredictionCollector(std::unique_ptr<DefaultPredictionStrategy> strategy);
 
   std::vector<Prediction> collect_predictions(const Forest& forest,
-                                              Data* train_data,
-                                              Data* data,
+                                              const Data& train_data,
+                                              const Data& data,
                                               const std::vector<std::vector<size_t>>& leaf_nodes_by_tree,
                                               const std::vector<std::vector<bool>>& valid_trees_by_sample,
                                               bool estimate_variance,
-                                              bool estimate_error);
+                                              bool estimate_error) const;
 
 private:
-  void validate_prediction(size_t sample, Prediction prediction);
+  void validate_prediction(size_t sample, const Prediction& prediction) const;
 
-  std::shared_ptr<DefaultPredictionStrategy> strategy;
+  std::unique_ptr<DefaultPredictionStrategy> strategy;
   SampleWeightComputer weight_computer;
 };
 
+} // namespace grf
 
-#endif //GRF_FULLPREDICTOR_H
+#endif //GRF_DEFAULTPREDICTIONCOLLECTOR_H
