@@ -17,14 +17,19 @@
 
 #include "relabeling/NoopRelabelingStrategy.h"
 
-std::unordered_map<size_t, double> NoopRelabelingStrategy::relabel(
+namespace grf {
+
+bool NoopRelabelingStrategy::relabel(
     const std::vector<size_t>& samples,
-    const Data* data) {
+    const Data& data,
+    std::vector<double>& responses_by_sample) const {
 
-  std::unordered_map<size_t, double> relabeled_observations;
   for (size_t sample : samples) {
-    double outcome = data->get_outcome(sample);
-    relabeled_observations[sample] = outcome;
+    double outcome = data.get_outcome(sample);
+    responses_by_sample[sample] = outcome;
   }
-  return relabeled_observations;
+  return false;
 }
+
+} // namespace grf
+
