@@ -23,41 +23,45 @@
 #include "commons/globals.h"
 #include "splitting/SplittingRule.h"
 
-class ProbabilitySplittingRule: public SplittingRule {
+namespace grf {
+
+class ProbabilitySplittingRule final: public SplittingRule {
 public:
-  ProbabilitySplittingRule(const Data* data,
+  ProbabilitySplittingRule(size_t max_num_unique_values,
                            size_t num_classes,
                            double alpha,
                            double imbalance_penalty);
   ~ProbabilitySplittingRule();
 
-  bool find_best_split(size_t node,
+  bool find_best_split(const Data& data,
+                       size_t node,
                        const std::vector<size_t>& possible_split_vars,
-                       const std::unordered_map<size_t, double>& labels_by_sample,
+                       const std::vector<double>& responses_by_sample,
                        const std::vector<std::vector<size_t>>& samples,
                        std::vector<size_t>& split_vars,
                        std::vector<double>& split_values);
 
 private:
-  void find_best_split_value_small_q(size_t node, size_t var, size_t num_classes, size_t* class_counts,
+  void find_best_split_value_small_q(const Data& data,
+                                     size_t node, size_t var, size_t num_classes, size_t* class_counts,
                                      size_t size_node,
                                      size_t min_child_size,
                                      double& best_value,
                                      size_t& best_var,
                                      double& best_decrease,
-                                     const std::unordered_map<size_t, double>& labels_by_sample,
+                                     const std::vector<double>& responses_by_sample,
                                      const std::vector<std::vector<size_t>>& samples);
 
-  void find_best_split_value_large_q(size_t node, size_t var, size_t num_classes, size_t* class_counts,
+  void find_best_split_value_large_q(const Data& data,
+                                     size_t node, size_t var, size_t num_classes, size_t* class_counts,
                                      size_t size_node,
                                      size_t min_child_size,
                                      double& best_value,
                                      size_t& best_var,
                                      double& best_decrease,
-                                     const std::unordered_map<size_t, double>& labels_by_sample,
+                                     const std::vector<double>& responses_by_sample,
                                      const std::vector<std::vector<size_t>>& samples);
 
-  const Data* data;
   size_t num_classes;
 
   double alpha;
@@ -69,4 +73,6 @@ private:
   DISALLOW_COPY_AND_ASSIGN(ProbabilitySplittingRule);
 };
 
+} // namespace grf
+
 #endif //GRF_PROBABILITYSPLITTINGRULE_H
