@@ -16,33 +16,33 @@
  #-------------------------------------------------------------------------------*/
 
 #include <algorithm>
-#include <math.h>
 
 #include "RegressionSplittingRule.h"
 
-RegressionSplittingRule::RegressionSplittingRule(const Data* data,
+namespace grf {
+
+RegressionSplittingRule::RegressionSplittingRule(size_t max_num_unique_values,
                                                  double alpha,
                                                  double imbalance_penalty):
-    data(data),
     alpha(alpha),
     imbalance_penalty(imbalance_penalty) {
-  size_t max_num_unique_values = data->get_max_num_unique_values();
   this->counter = new size_t[max_num_unique_values];
   this->sums = new double[max_num_unique_values];
 }
 
 RegressionSplittingRule::~RegressionSplittingRule() {
-  if (counter != 0) {
+  if (counter != nullptr) {
     delete[] counter;
   }
-  if (sums != 0) {
+  if (sums != nullptr) {
     delete[] sums;
   }
 }
 
-bool RegressionSplittingRule::find_best_split(size_t node,
+bool RegressionSplittingRule::find_best_split(const Data& data,
+                                              size_t node,
                                               const std::vector<size_t>& possible_split_vars,
-                                              const std::unordered_map<size_t, double>& labels_by_sample,
+                                              const std::vector<double>& responses_by_sample,
                                               const std::vector<std::vector<size_t>>& samples,
                                               std::vector<size_t>& split_vars,
                                               std::vector<double>& split_values) {
@@ -53,7 +53,7 @@ bool RegressionSplittingRule::find_best_split(size_t node,
   // Precompute the sum of outcomes in this node.
   double sum_node = 0.0;
   for (auto& sample : samples[node]) {
-    sum_node += labels_by_sample.at(sample);
+    sum_node += responses_by_sample[sample];
   }
 
   // Initialize the variables to track the best split variable.
@@ -64,13 +64,13 @@ bool RegressionSplittingRule::find_best_split(size_t node,
   // For all possible split variables
   for (auto& var : possible_split_vars) {
     // Use faster method for both cases
-    double q = (double) size_node / (double) data->get_num_unique_data_values(var);
+    double q = (double) size_node / (double) data.get_num_unique_data_values(var);
     if (q < Q_THRESHOLD) {
-      find_best_split_value_small_q(node, var, sum_node, size_node, min_child_size,
-                                    best_value, best_var, best_decrease, labels_by_sample, samples);
+      find_best_split_value_small_q(data, node, var, sum_node, size_node, min_child_size,
+                                    best_value, best_var, best_decrease, responses_by_sample, samples);
     } else {
-      find_best_split_value_large_q(node, var, sum_node, size_node, min_child_size,
-                                    best_value, best_var, best_decrease, labels_by_sample, samples);
+      find_best_split_value_large_q(data, node, var, sum_node, size_node, min_child_size,
+                                    best_value, best_var, best_decrease, responses_by_sample, samples);
     }
   }
 
@@ -85,16 +85,17 @@ bool RegressionSplittingRule::find_best_split(size_t node,
   return false;
 }
 
-void RegressionSplittingRule::find_best_split_value_small_q(size_t node, size_t var,
+void RegressionSplittingRule::find_best_split_value_small_q(const Data& data,
+                                                            size_t node, size_t var,
                                                             double sum_node,
                                                             size_t size_node,
                                                             size_t min_child_size,
                                                             double& best_value, size_t& best_var,
                                                             double& best_decrease,
-                                                            const std::unordered_map<size_t, double>& labels_by_sample,
+                                                            const std::vector<double>& responses_by_sample,
                                                             const std::vector<std::vector<size_t>>& samples) {
   std::vector<double> possible_split_values;
-  data->get_all_values(possible_split_values, samples.at(node), var);
+  data.get_all_values(possible_split_values, samples[node], var);
 
   // Try next variable if all equal for this
   if (possible_split_values.size() < 2) {
@@ -115,8 +116,8 @@ void RegressionSplittingRule::find_best_split_value_small_q(size_t node, size_t
 
   // Sum in right child and possible split
   for (auto& sample : samples[node]) {
-    double value = data->get(sample, var);
-    double response = labels_by_sample.at(sample);
+    double value = data.get(sample, var);
+    double response = responses_by_sample[sample];
 
     // Count samples until split_value reached
     for (size_t i = 0; i < num_splits; ++i) {
@@ -161,7 +162,8 @@ void RegressionSplittingRule::find_best_split_value_small_q(size_t node, size_t
   }
 }
 
-void RegressionSplittingRule::find_best_split_value_large_q(size_t node,
+void RegressionSplittingRule::find_best_split_value_large_q(const Data& data,
+                                                            size_t node,
                                                             size_t var,
                                                             double sum_node,
                                                             size_t size_node,
@@ -169,17 +171,17 @@ void RegressionSplittingRule::find_best_split_value_large_q(size_t node,
                                                             double& best_value,
                                                             size_t& best_var,
                                                             double& best_decrease,
-                                                            const std::unordered_map<size_t, double>& responses_by_sample,
+                                                            const std::vector<double>& responses_by_sample,
                                                             const std::vector<std::vector<size_t>>& samples) {
   // Set counters to 0
-  size_t num_unique = data->get_num_unique_data_values(var);
+  size_t num_unique = data.get_num_unique_data_values(var);
   std::fill(counter, counter + num_unique, 0);
   std::fill(sums, sums + num_unique, 0);
 
   for (auto& sample : samples[node]) {
-    size_t index = data->get_index(sample, var);
+    size_t index = data.get_index(sample, var);
 
-    sums[index] += responses_by_sample.at(sample);
+    sums[index] += responses_by_sample[sample];
     ++counter[index];
   }
 
@@ -211,9 +213,11 @@ void RegressionSplittingRule::find_best_split_value_large_q(size_t node,
 
     // If better than before, use this
     if (decrease > best_decrease) {
-      best_value = data->get_unique_data_value(var, i);
+      best_value = data.get_unique_data_value(var, i);
       best_var = var;
       best_decrease = decrease;
     }
   }
 }
+
+} // namespace grf
