@@ -18,28 +18,31 @@
 #ifndef GRF_REGRESSIONSPLITTINGRULE_H
 #define GRF_REGRESSIONSPLITTINGRULE_H
 
-#include "tree/Tree.h"
-#include "splitting/SplittingRule.h"
-#include <unordered_map>
 #include "commons/DefaultData.h"
+#include "splitting/SplittingRule.h"
+#include "tree/Tree.h"
 
-class RegressionSplittingRule: public SplittingRule {
+namespace grf {
+
+class RegressionSplittingRule final: public SplittingRule {
 public:
-  RegressionSplittingRule(const Data* data,
+  RegressionSplittingRule(size_t max_num_unique_values,
                           double alpha,
                           double imbalance_penalty);
 
   ~RegressionSplittingRule();
 
-  bool find_best_split(size_t node,
+  bool find_best_split(const Data& data,
+                       size_t node,
                        const std::vector<size_t>& possible_split_vars,
-                       const std::unordered_map<size_t, double>& labels_by_sample,
+                       const std::vector<double>& responses_by_sample,
                        const std::vector<std::vector<size_t>>& samples,
                        std::vector<size_t>& split_vars,
                        std::vector<double>& split_values);
 
 private:
-  void find_best_split_value_small_q(size_t node,
+  void find_best_split_value_small_q(const Data& data,
+                                     size_t node,
                                      size_t var,
                                      double sum_node,
                                      size_t size_node,
@@ -47,9 +50,10 @@ private:
                                      double& best_value,
                                      size_t& best_var,
                                      double& best_decrease,
-                                     const std::unordered_map<size_t, double>& labels_by_sample,
+                                     const std::vector<double>& responses_by_sample,
                                      const std::vector<std::vector<size_t>>& samples);
-  void find_best_split_value_large_q(size_t node,
+  void find_best_split_value_large_q(const Data& data,
+                                     size_t node,
                                      size_t var,
                                      double sum_node,
                                      size_t size_node,
@@ -57,10 +61,9 @@ private:
                                      double& best_value,
                                      size_t& best_var,
                                      double& best_decrease,
-                                     const std::unordered_map<size_t, double>& responses_by_sample,
+                                     const std::vector<double>& responses_by_sample,
                                      const std::vector<std::vector<size_t>>& samples);
 
-  const Data* data;
   size_t* counter;
   double* sums;
 
@@ -70,5 +73,6 @@ private:
   DISALLOW_COPY_AND_ASSIGN(RegressionSplittingRule);
 };
 
+} // namespace grf
 
 #endif //GRF_REGRESSIONSPLITTINGRULE_H
