@@ -15,16 +15,22 @@
   along with grf. If not, see <http://www.gnu.org/licenses/>.
  #-------------------------------------------------------------------------------*/
 
-#include <unordered_map>
 #include "splitting/SplittingRule.h"
 #include "splitting/factory/ProbabilitySplittingRuleFactory.h"
 #include "splitting/ProbabilitySplittingRule.h"
 
+namespace grf {
+
 ProbabilitySplittingRuleFactory::ProbabilitySplittingRuleFactory(size_t num_classes):
     num_classes(num_classes) {}
 
-std::shared_ptr<SplittingRule> ProbabilitySplittingRuleFactory::create(const Data* data,
-                                                                       const TreeOptions& options) {
-  return std::shared_ptr<SplittingRule>(new ProbabilitySplittingRule(
-      data, num_classes, options.get_alpha(), options.get_imbalance_penalty()));
+std::unique_ptr<SplittingRule> ProbabilitySplittingRuleFactory::create(const Data& data,
+                                                                       const TreeOptions& options) const {
+  return std::unique_ptr<SplittingRule>(new ProbabilitySplittingRule(
+      data.get_max_num_unique_values(),
+      num_classes,
+      options.get_alpha(),
+      options.get_imbalance_penalty()));
 }
+
+} // namespace grf
