@@ -28,13 +28,15 @@
 #include "tree/Tree.h"
 #include "tree/TreeOptions.h"
 
+namespace grf {
+
 class TreeTrainer {
 public:
-  TreeTrainer(std::shared_ptr<RelabelingStrategy> relabeling_strategy,
-              std::shared_ptr<SplittingRuleFactory> splitting_rule_factory,
-              std::shared_ptr<OptimizedPredictionStrategy> prediction_strategy);
+  TreeTrainer(std::unique_ptr<RelabelingStrategy> relabeling_strategy,
+              std::unique_ptr<SplittingRuleFactory> splitting_rule_factory,
+              std::unique_ptr<OptimizedPredictionStrategy> prediction_strategy);
 
-  std::shared_ptr<Tree> train(const Data* data,
+  std::unique_ptr<Tree> train(const Data& data,
                               RandomSampler& sampler,
                               const std::vector<size_t>& clusters,
                               const TreeOptions& options) const;
@@ -45,39 +47,44 @@ private:
                          std::vector<size_t>& split_vars,
                          std::vector<double>& split_values) const;
 
-  void repopulate_leaf_nodes(std::shared_ptr<Tree> tree,
-                             const Data* data,
-                             const std::vector<size_t> &leaf_samples) const;
+  void repopulate_leaf_nodes(const std::unique_ptr<Tree>& tree,
+                             const Data& data,
+                             const std::vector<size_t>& leaf_samples,
+                             const bool prune_empty_leaves) const;
 
   void create_split_variable_subset(std::vector<size_t>& result,
                                     RandomSampler& sampler,
-                                    const Data* data,
+                                    const Data& data,
                                     uint mtry) const;
 
   bool split_node(size_t node,
-                  const Data* data,
-                  std::shared_ptr<SplittingRule> splitting_rule,
+                  const Data& data,
+                  const std::unique_ptr<SplittingRule>& splitting_rule,
                   RandomSampler& sampler,
                   std::vector<std::vector<size_t>>& child_nodes,
                   std::vector<std::vector<size_t>>& samples,
                   std::vector<size_t>& split_vars,
                   std::vector<double>& split_values,
+                  std::vector<double>& responses_by_sample,
                   const TreeOptions& tree_options) const;
 
   bool split_node_internal(size_t node,
-                           const Data* data,
-                           std::shared_ptr<SplittingRule> splitting_rule,
+                           const Data& data,
+                           const std::unique_ptr<SplittingRule>& splitting_rule,
                            const std::vector<size_t>& possible_split_vars,
-                           std::vector<std::vector<size_t>>& samples,
+                           const std::vector<std::vector<size_t>>& samples,
                            std::vector<size_t>& split_vars,
                            std::vector<double>& split_values,
+                           std::vector<double>& responses_by_sample,
                            uint min_node_size) const ;
 
   std::set<size_t> disallowed_split_variables;
 
-  std::shared_ptr<RelabelingStrategy> relabeling_strategy;
-  std::shared_ptr<SplittingRuleFactory> splitting_rule_factory;
-  std::shared_ptr<OptimizedPredictionStrategy> prediction_strategy;
+  std::unique_ptr<RelabelingStrategy> relabeling_strategy;
+  std::unique_ptr<SplittingRuleFactory> splitting_rule_factory;
+  std::unique_ptr<OptimizedPredictionStrategy> prediction_strategy;
 };
 
+} // namespace grf
+
 #endif //GRF_TREETRAINER_H
